@@ -198,7 +198,7 @@ static int parse_interface(libusb_context *ctx,
 	while (size >= INTERFACE_DESC_LENGTH) {
 		struct libusb_interface_descriptor *altsetting =
 			(struct libusb_interface_descriptor *) usb_interface->altsetting;
-		altsetting = realloc(altsetting,
+		altsetting = usbi_reallocf(altsetting,
 			sizeof(struct libusb_interface_descriptor) *
 			(usb_interface->num_altsetting + 1));
 		if (!altsetting) {
@@ -229,6 +229,12 @@ static int parse_interface(libusb_context *ctx,
 					header.bLength);
 				r = LIBUSB_ERROR_IO;
 				goto err;
+			} else if (header.bLength > size) {
+				usbi_warn(ctx, "invalid descriptor of length %d",
+					header.bLength);
+				/* The remaining bytes are bogus, but at least
+				 * one interface is OK, so let's continue. */
+				break;
 			}
 
 			/* If we find another "proper" descriptor then we're done */
@@ -371,6 +377,14 @@ static int parse_configuration(struct libusb_context *ctx,
 		while (size >= DESC_HEADER_LENGTH) {
 			usbi_parse_descriptor(buffer, "bb", &header, 0);
 
+			/* If we've parsed at least one config descriptor then
+			 * let's return that. */
+			if (header.bLength > size && i) {
+				usbi_warn(ctx, "invalid descriptor length of %d",
+					header.bLength);
+				return size;
+			}
+
 			if ((header.bLength > size) ||
 					(header.bLength < DESC_HEADER_LENGTH)) {
 				usbi_err(ctx, "invalid descriptor length of %d",
@@ -486,8 +500,10 @@ int API_EXPORTED libusb_get_active_config_descriptor(libusb_device *dev,
 	if (r < 0)
 		goto err;
 
+	_config->wTotalLength = 0;
 	usbi_parse_descriptor(tmp, "bbw", _config, host_endian);
-	buf = malloc(_config->wTotalLength);
+	if (_config->wTotalLength != 0)
+		buf = malloc(_config->wTotalLength);
 	if (!buf) {
 		r = LIBUSB_ERROR_NO_MEM;
 		goto err;
@@ -718,7 +734,7 @@ int API_EXPORTED libusb_get_string_descriptor_ascii(libusb_device_handle *dev,
 		if (di >= (length - 1))
 			break;
 
-		if (tbuf[si + 1]) /* high byte */
+		if ((tbuf[si] & 0x80) || (tbuf[si + 1])) /* non-ASCII */
 			data[di++] = '?';
 		else
 			data[di++] = tbuf[si];
